<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Gop10001SimulacaoValoresController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $parameterBag;
    protected $util;
    protected $environment;
    protected $dataSourceName;
    protected $parameterBagColums;
    protected $dataSourceManager;
    protected $service;
    
    public function __construct($crudService, $parameterBag, $service, $util, $environment, $dataSourceManager){
        $this->crudService          = $crudService;
        $this->service              = $service;
        $this->util                 = $util;
        $this->environment          = $environment;
        $this->dataSourceManager    = $dataSourceManager;
    }
   
    public function filterSimulation(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try {
            if(isset($row['NRLISTA']) && $row['NRLISTA'] != '') {
                $this->util->deleteLista($row['NRLISTA']);
            }
            $nrLista = $this->util->insertLista($row['CDFILIAL']);
            $conditions = array(
                array(
                    "columnName"=>"P_NRLISTA", 
                    "operator"=>"=", 
                    "value"=>$nrLista
                ),
                array(
                    "columnName"=>"P_IDPERIODOORCA", 
                    "operator"=>"=", 
                    "value"=>$row["P_IDPERIODOORCA"]
                ),
                array(
                    "columnName"=>"P_NRPERIODOORCA", 
                    "operator"=>"=", 
                    "value"=>$row["P_NRPERIODOORCA"]
                ),
                array(
                    "columnName"=>"P_NRANOORCAGOP", 
                    "operator"=>"=", 
                    "value"=>$row["P_NRANOORCAGOP"]
                ),
                array(
                    "columnName"=>"P_NRORG", 
                    "operator"=>"=", 
                    "value"=>$this->environment->getNrOrg()
                )
            );
            $result = $this->crudService->getFilteredDataSet('gop10001_simulacaoValores', $conditions);
            $dados = $result->getRows();
            $dataSet = array(
                "data"=>$dados,
                "nrLista"=>$nrLista
            );
            $response->addDataSet(new DataSet('success', $dataSet));
        }catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500)); 
        }
    }
    
    public function verifyParamVrFaturamento(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try{
            if(!is_string($row["CDFILIAL"]) && count($row["CDFILIAL"]) > 1){
                $row["CDFILIAL"] = "P";
            }
            
            $paramVrFaturamento = $this->service->verifyParamVrFaturamento($row);
            $dataSet = array (
                'VRFATURAMENTO' => $paramVrFaturamento
            );
            $response->addDataSet(new DataSet('result', $dataSet));
            
        }catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveCalculation(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try{
            $this->service->beginTransaction();

            $this->updateCalc($row['NRLISTA'], $row['CODIGO'], $row['DATAMONTH'], $row['ITEMSCHANGED']);
            
            $this->service->commit();
        }catch(\Exception $e) {
            $this->service->rollback();
            
            $response->setError(new Error ($e->getMessage(), 500));
        }
    }
    
    private function updateCalc( $nrLista, $codigo, $dataMonth, $itemsChanged) {
        $codigo = $codigo ? "BUDGETCHANGED" : null;
        
        if($codigo == 'BUDGETCHANGED'){
            foreach($itemsChanged as $row){
                $percent = array_key_exists('ACCUMULATOR_RATEIO', $row) && $row['ACCUMULATOR_RATEIO'] == 1 ? $row['RATEIO'] :
                    (array_key_exists('ACCUMULATOR_RATEIO', $row) ? $row['ACCUMULATOR_RATEIO'] : null);
                
                if(isset($row['RATEIO_ZERADO_MONTH']) && !isset($row['RATEIO'])){
                    $percent = 0;
                }
                
                $this->service->updateSimulation($nrLista, $percent, $codigo, $row, null);
            }
        }else {
            foreach($dataMonth as $rowMonth) {
                $this->service->updateSimulation(null, null, null, null, $rowMonth);
            }
        }
       
    }
    
    public function saveCalculationSimulation(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try{
            $this->service->saveSimulation();
        }catch(Exception $e){
            $response->setError(new Error ($e->getMessage(), 500));
        }
    }
}